@@ -1,35 +1,31 @@
 """Containers are components that contain other components."""
 
 from gdsfactory.add_padding import add_padding_to_size, add_padding_to_size_container
-from gdsfactory.components.add_fiber_array_optical_south_electrical_north import (
+from gdsfactory.components import (
     add_fiber_array_optical_south_electrical_north,
-)
-from gdsfactory.components.add_termination import add_termination
-from gdsfactory.components.add_trenches import add_trenches, add_trenches90
-from gdsfactory.components.array_component import array
-from gdsfactory.components.cutback_2x2 import cutback_2x2
-from gdsfactory.components.cutback_bend import (
+    add_termination,
+    add_trenches,
+    add_trenches90,
+    array,
     cutback_bend,
     cutback_bend90,
     cutback_bend90circular,
     cutback_bend180,
     cutback_bend180circular,
-    staircase,
-)
-from gdsfactory.components.cutback_component import (
     cutback_component,
     cutback_component_mirror,
-)
-from gdsfactory.components.cutback_loss import (
     cutback_loss,
     cutback_loss_bend90,
     cutback_loss_bend180,
     cutback_loss_mmi1x2,
     cutback_loss_spirals,
+    cutback_splitter,
+    extend_ports,
+    pack_doe,
+    pack_doe_grid,
+    staircase,
 )
-from gdsfactory.components.cutback_splitter import cutback_splitter
-from gdsfactory.components.extension import extend_ports
-from gdsfactory.components.pack_doe import pack_doe, pack_doe_grid
+from gdsfactory.components.pcms.cutback_2x2 import cutback_2x2
 from gdsfactory.functions import (
     extract,
     move_port_to_zero,
