@@ -5,7 +5,7 @@ Adapted from PHIDL https://github.com/amccaugh/phidl/ by Adam McCaughan
 
 from __future__ import annotations
 
-import freetype  # type: ignore
+import freetype
 import numpy as np
 import numpy.typing as npt
 from matplotlib import font_manager
@@ -17,7 +17,7 @@ from gdsfactory.component import Component
 _cached_fonts: dict[str, freetype.Face] = {}
 
 try:
-    import freetype  # type: ignore
+    import freetype
 except ImportError:
     print(
         "gdsfactory requires freetype to use real fonts. "
@@ -78,7 +78,7 @@ def _get_glyph(font: freetype.Face, letter: str) -> tuple[Component, float, floa
         font.gds_glyphs = {}
 
     if letter in font.gds_glyphs:
-        return font.gds_glyphs[letter]  # type: ignore
+        return font.gds_glyphs[letter]  # type: ignore[no-any-return]
 
     # Get the font name
     font_name = font.family_name.decode().replace(" ", "_")
@@ -160,18 +160,16 @@ def _get_glyph(font: freetype.Face, letter: str) -> tuple[Component, float, floa
         c2.add_polygon(np.array(p), layer=(1, 0))
     if orientation == 0:
         # TrueType specification, fill the clockwise contour
-        component = boolean(c1, c2, operation="not")
-    elif orientation == 1:
-        # PostScript specification, fill the counterclockwise contour
-        component = boolean(c2, c1, operation="not")
+        component = boolean(c1, c2, operation="not", layer=(1, 0))
     else:
-        raise ValueError(f"Unknown orientation {orientation} for letter {letter}")
+        # PostScript specification, fill the counterclockwise contour
+        component = boolean(c2, c1, operation="not", layer=(1, 0))
 
     component.name = block_name
 
     # Cache the return value and return it
     font.gds_glyphs[letter] = (component, glyph.advance.x, font.size.ascender)
-    return font.gds_glyphs[letter]  # type: ignore
+    return font.gds_glyphs[letter]  # type: ignore[no-any-return]
 
 
 def _polygon_orientation(vertices: npt.NDArray[np.float64]) -> int:
@@ -189,7 +187,7 @@ def _polygon_orientation(vertices: npt.NDArray[np.float64]) -> int:
 
 
 if __name__ == "__main__":
-    from gdsfactory.components.text_freetype import text_freetype
+    from gdsfactory.components import text_freetype
 
-    c = text_freetype("hello")
+    c = text_freetype("abcd")
     c.show()
