@@ -1,8 +1,9 @@
 from __future__ import annotations
 
 import warnings
+from collections.abc import Callable, Sequence
 from functools import partial
-from typing import TYPE_CHECKING, Literal
+from typing import TYPE_CHECKING, Any, Literal, TypeAlias
 
 import kfactory as kf
 import numpy as np
@@ -12,7 +13,7 @@ from numpy import cos, float64, sin
 import gdsfactory as gf
 
 if TYPE_CHECKING:
-    from gdsfactory.component import Component, Instance
+    from gdsfactory.component import Component, ComponentReference
     from gdsfactory.typings import LayerSpec, LayerSpecs
 
 RAD2DEG = 180.0 / np.pi
@@ -40,8 +41,8 @@ def move_port_to_zero(
     if mirror:
         ref.dmirror()
 
-    movement = np.array(ref.ports[port_name].dcenter)
-    ref.dmove(-movement)
+    movement = np.array(ref.ports[port_name].center)
+    ref.move(tuple(-movement))
     c.add_ports(ref.ports)
     c.copy_child_info(component)
     return c
@@ -70,27 +71,29 @@ def extract(
     Args:
         component: to extract the layers from.
         layers: list of layers to extract.
-        recursive: if True, extracts layers recursively and returns a flattened Component.
+        recursive: if True, extracts the shapes recursively.
     """
     from gdsfactory.pdk import get_layer_tuple
 
-    c = component.dup()
-    if recursive:
-        c.flatten()
+    c = gf.Component()
 
     layer_tuples = [get_layer_tuple(layer) for layer in layers]
-    component_layers = get_layers(c)
+    component_layers = get_layers(component)
 
     for layer_tuple in layer_tuples:
         if layer_tuple not in component_layers:
             warnings.warn(
-                f"Layer {layer_tuple} not found in component {component.name!r} layers."
+                f"Layer {layer_tuple} not found in component {component.name!r} layers. {component_layers}",
+                stacklevel=3,
             )
 
     for layer_tuple in component_layers:
-        if layer_tuple not in layer_tuples:
+        if layer_tuple in layer_tuples:
             layer_index = c.kcl.layer(*layer_tuple)
-            c.shapes(layer_index).clear()
+            if recursive:
+                c.shapes(layer_index).insert(component.begin_shapes_rec(layer_index))
+            else:
+                c.shapes(layer_index).insert(component.shapes(layer_index))
 
     return c
 
@@ -98,10 +101,7 @@ def extract(
 def move_to_center(component: Component, dx: float = 0, dy: float = 0) -> gf.Component:
     """Moves the component to the center of the bounding box."""
     c = component
-    c.transform(
-        gf.kdb.DTrans(-c.dbbox().center().x + dx, -c.dbbox().center().y + dy),
-        no_warn=True,
-    )
+    c.transform(gf.kdb.DTrans(-c.dbbox().center().x + dx, -c.dbbox().center().y + dy))
     return c
 
 
@@ -119,19 +119,20 @@ def move_port(
         dy: to move the port.
     """
     c = component
-    c.transform(
-        gf.kdb.DTrans(-c.ports[port_name].dx + dx, -c.ports[port_name].dy + dy),
-        no_warn=True,
-    )
+    c.transform(gf.kdb.DTrans(-c.ports[port_name].x + dx, -c.ports[port_name].y + dy))
     return c
 
 
+GetPolygonsResult: TypeAlias = "dict[LayerSpec, list[kf.kdb.Polygon]]"
+
+
 def get_polygons(
-    component_or_instance: Component | Instance,
+    component_or_instance: "Component | ComponentReference",
     merge: bool = False,
     by: Literal["index", "name", "tuple"] = "index",
     layers: LayerSpecs | None = None,
-) -> dict[tuple[int, int] | str | int, list[kf.kdb.Polygon]]:
+    smooth: float | None = None,
+) -> GetPolygonsResult:
     """Returns a dict of Polygons per layer.
 
     Args:
@@ -139,11 +140,12 @@ def get_polygons(
         merge: if True, merges the polygons.
         by: the format of the resulting keys in the dictionary ('index', 'name', 'tuple').
         layers: list of layer specs to extract the polygons from. If None, extracts all layers.
+        smooth: if True, smooths the polygons.
     """
     from gdsfactory.pdk import get_layer, get_layer_name, get_layer_tuple
 
     if by == "index":
-        get_key = get_layer
+        get_key: "Callable[[LayerSpec], LayerSpec]" = get_layer
     elif by == "name":
         get_key = get_layer_name
     elif by == "tuple":
@@ -151,7 +153,7 @@ def get_polygons(
     else:
         raise ValueError("argument 'by' should be 'index' | 'name' | 'tuple'")
 
-    polygons: dict[tuple[int, int] | str | int, list[kf.kdb.Polygon]] = {}
+    polygons: GetPolygonsResult = {}
 
     c = component_or_instance
     if layers is None:
@@ -161,18 +163,20 @@ def get_polygons(
             if not c.bbox(c.kcl.layer(info)).empty()
         ]
 
-    layer_indexes = [gf.get_layer(layer) for layer in layers]
+    layer_indexes = [get_layer(layer) for layer in layers]
 
     for layer_index in layer_indexes:
         layer_key = get_key(layer_index)
         if isinstance(component_or_instance, gf.Component):
-            r = gf.kdb.Region(c.begin_shapes_rec(layer_index))
+            r = gf.Region(c.begin_shapes_rec(layer_index))
         else:
             r = kf.kdb.Region(c.cell.begin_shapes_rec(layer_index)).transformed(
                 c.cplx_trans
             )
         if layer_key not in polygons:
             polygons[layer_key] = []
+        if smooth:
+            r.smooth(round(smooth / c.kcl.dbu))
         if merge:
             r.merge()
         for p in r.each():
@@ -181,12 +185,12 @@ def get_polygons(
 
 
 def get_polygons_points(
-    component_or_instance: Component | Instance,
+    component_or_instance: "Component | ComponentReference",
     merge: bool = False,
     scale: float | None = None,
-    by: Literal["index"] | Literal["name"] | Literal["tuple"] = "index",
+    by: Literal["index", "name", "tuple"] = "index",
     layers: LayerSpecs | None = None,
-) -> dict[int | str | tuple[int, int], list[tuple[float, float]]]:
+) -> dict[int | str | tuple[int, int], list[npt.NDArray[np.floating[Any]]]]:
     """Returns a dict with list of points per layer.
 
     Args:
@@ -199,9 +203,11 @@ def get_polygons_points(
     polygons_dict = get_polygons(
         component_or_instance=component_or_instance, merge=merge, by=by, layers=layers
     )
-    polygons_points: dict[tuple[int, int] | str | int, list[tuple[float, float]]] = {}
+    polygons_points: dict[
+        tuple[int, int] | str | int, list[npt.NDArray[np.floating[Any]]]
+    ] = {}
     for layer, polygons in polygons_dict.items():
-        all_points = []
+        all_points: list[npt.NDArray[np.floating[Any]]] = []
         for polygon in polygons:
             if scale:
                 points = np.array(
@@ -227,8 +233,8 @@ def get_polygons_points(
 
 
 def get_point_inside(
-    component_or_instance: Component | Instance, layer: LayerSpec
-) -> np.ndarray:
+    component_or_instance: "Component | ComponentReference", layer: LayerSpec
+) -> npt.NDArray[np.floating[Any]]:
     """Returns a point inside the component or instance.
 
     Args:
@@ -236,37 +242,39 @@ def get_point_inside(
         layer: to find a point inside.
     """
     layer = gf.get_layer(layer)
-    return get_polygons_points(component_or_instance, layers=[layer])[layer][0][0]
+    return np.array(
+        get_polygons_points(component_or_instance, layers=[layer])[layer][0][0]
+    )
 
 
-def sign_shape(pts: npt.NDArray[np.float64]) -> float64:
+def sign_shape(pts: npt.NDArray[np.floating[Any]]) -> float:
     pts2 = np.roll(pts, 1, axis=0)
     dx = pts2[:, 0] - pts[:, 0]
     y = pts2[:, 1] + pts[:, 1]
-    return np.sign((dx * y).sum())
+    return float(np.sign((dx * y).sum()))
 
 
-def area(pts: npt.NDArray[np.float64]) -> float64:
+def area(pts: npt.NDArray[np.floating[Any]]) -> float:
     """Returns the area."""
     pts2 = np.roll(pts, 1, axis=0)
     dx = pts2[:, 0] - pts[:, 0]
     y = pts2[:, 1] + pts[:, 1]
-    return (dx * y).sum() / 2
+    return float(np.sum(dx * y) / 2)
 
 
-def centered_diff(a: npt.NDArray[np.float64]) -> npt.NDArray[np.float64]:
+def centered_diff(a: npt.NDArray[np.floating[Any]]) -> npt.NDArray[np.floating[Any]]:
     d = (np.roll(a, -1, axis=0) - np.roll(a, 1, axis=0)) / 2
-    return d[1:-1]
+    return np.array(d)[1:-1]
 
 
-def centered_diff2(a: npt.NDArray[np.float64]) -> npt.NDArray[np.float64]:
+def centered_diff2(a: npt.NDArray[np.floating[Any]]) -> npt.NDArray[np.floating[Any]]:
     d = (np.roll(a, -1, axis=0) - a) - (a - np.roll(a, 1, axis=0))
-    return d[1:-1]
+    return np.array(d[1:-1])
 
 
 def curvature(
-    points: npt.NDArray[np.float64], t: npt.NDArray[np.float64]
-) -> npt.NDArray[np.float64]:
+    points: npt.NDArray[np.floating[Any]], t: npt.NDArray[np.floating[Any]]
+) -> npt.NDArray[np.floating[Any]]:
     """Args are the points and the tangents at each point.
 
         points : numpy.array shape (n, 2)
@@ -292,16 +300,18 @@ def curvature(
     dx2 = dp2[:, 0] / dt**2
     dy2 = dp2[:, 1] / dt**2
 
-    return (dx * dy2 - dx2 * dy) / (dx**2 + dy**2) ** (3 / 2)
+    res = (dx * dy2 - dx2 * dy) / (dx**2 + dy**2) ** (3 / 2)
+    assert isinstance(res, np.ndarray)
+    return res
 
 
 def radius_of_curvature(
-    points: npt.NDArray[np.float64], t: npt.NDArray[np.float64]
-) -> float64:
+    points: npt.NDArray[np.floating[Any]], t: npt.NDArray[np.floating[Any]]
+) -> npt.NDArray[np.floating[Any]]:
     return 1 / curvature(points, t)
 
 
-def path_length(points: npt.NDArray[np.float64]) -> float64:
+def path_length(points: npt.NDArray[np.floating[Any]]) -> float:
     """Returns: The path length.
 
     Args:
@@ -309,10 +319,10 @@ def path_length(points: npt.NDArray[np.float64]) -> float64:
     """
     dpts = points[1:, :] - points[:-1, :]
     _d = dpts**2
-    return np.sum(np.sqrt(_d[:, 0] + _d[:, 1]))
+    return float(np.sum(np.sqrt(_d[:, 0] + _d[:, 1])))
 
 
-def snap_angle(a: float64) -> int:
+def snap_angle(a: float) -> float:
     """Returns angle snapped along manhattan angle (0, 90, 180, 270).
 
     a: angle in deg
@@ -331,29 +341,27 @@ def snap_angle(a: float64) -> int:
         return 0
 
 
-def angles_rad(pts: npt.NDArray[np.float64]) -> npt.NDArray[np.float64]:
+def angles_rad(pts: npt.NDArray[np.floating[Any]]) -> npt.NDArray[np.floating[Any]]:
     """Returns the angles (radians) of the connection between each point and the next."""
     _pts = np.roll(pts, -1, 0)
-    return np.arctan2(_pts[:, 1] - pts[:, 1], _pts[:, 0] - pts[:, 0])
+    return np.array(np.arctan2(_pts[:, 1] - pts[:, 1], _pts[:, 0] - pts[:, 0]))
 
 
-def angles_deg(pts: npt.NDArray[np.float64]) -> npt.NDArray[np.float64]:
+def angles_deg(pts: npt.NDArray[np.floating[Any]]) -> npt.NDArray[np.floating[Any]]:
     """Returns the angles (degrees) of the connection between each point and the next."""
     return angles_rad(pts) * RAD2DEG
 
 
 def extrude_path(
-    points: npt.NDArray[np.float64],
+    points: npt.NDArray[np.floating[Any]],
     width: float,
     with_manhattan_facing_angles: bool = True,
     spike_length: float64 | int | float = 0,
     start_angle: int | None = None,
     end_angle: int | None = None,
     grid: float | None = None,
-) -> npt.NDArray[np.float64]:
-    """Deprecated. Use gdsfactory.path.Path.extrude() instead.
-
-    Extrude a path of `width` along a curve defined by `points`.
+) -> npt.NDArray[np.floating[Any]]:
+    """Extrude a path of `width` along a curve defined by `points`.
 
     Args:
         points: numpy 2D array of shape (N, 2).
@@ -369,6 +377,8 @@ def extrude_path(
     """
     grid = grid or gf.kcl.dbu
 
+    assert grid is not None
+
     if isinstance(points, list):
         points = np.stack([(p[0], p[1]) for p in points], axis=0)
 
@@ -380,14 +390,17 @@ def extrude_path(
         _start_angle = a[0] + 180
         _end_angle = a[-2]
 
-    start_angle = start_angle if start_angle is not None else _start_angle
-    end_angle = end_angle if end_angle is not None else _end_angle
+    start_angle_ = start_angle if start_angle is not None else _start_angle
+    end_angle_ = end_angle if end_angle is not None else _end_angle
+
+    assert start_angle_ is not None
+    assert end_angle_ is not None
 
     a2 = angles_rad(points) * 0.5
     a1 = np.roll(a2, 1)
 
-    a2[-1] = end_angle * DEG2RAD - a2[-2]
-    a1[0] = start_angle * DEG2RAD - a1[1]
+    a2[-1] = end_angle_ * DEG2RAD - a2[-2]
+    a1[0] = start_angle_ * DEG2RAD - a1[1]
 
     a_plus = a2 + a1
     cos_a_min = np.cos(a2 - a1)
@@ -398,8 +411,8 @@ def extrude_path(
     points_back = np.flipud(points - offsets)
     if spike_length != 0:
         d = spike_length
-        a_start = start_angle * DEG2RAD
-        a_end = end_angle * DEG2RAD
+        a_start = start_angle_ * DEG2RAD
+        a_end = end_angle_ * DEG2RAD
         p_start_spike = points[0] + d * np.array([[cos(a_start), sin(a_start)]])
         p_end_spike = points[-1] + d * np.array([[cos(a_end), sin(a_end)]])
 
@@ -407,12 +420,12 @@ def extrude_path(
     else:
         pts = np.vstack((points + offsets, points_back))
 
-    return np.round(pts / grid) * grid
+    return np.array(np.round(pts / grid) * grid)
 
 
 def trim(
     component: Component,
-    domain: list[tuple[float, float]],
+    domain: Sequence[tuple[float, float]],
     flatten: bool = False,
 ) -> gf.Component:
     """Trim a component by another geometry, preserving the component's layers and ports.
@@ -448,7 +461,7 @@ def rotate(component: Component, angle: float) -> gf.Component:
 
     Args:
         component: to rotate.
-        angle: in degrees.
+        angle: in increments of 90°.
 
     Returns: Rotated component.
     """
@@ -466,24 +479,29 @@ rotate180 = partial(rotate, angle=180)
 rotate270 = partial(rotate, angle=270)
 
 
+@gf.cell
+def mirror(component: Component, x_mirror: bool = True) -> gf.Component:
+    """Rotate a component by an angle in degrees.
+
+    Args:
+        component: to rotate.
+        x_mirror: if True, mirrors the component along the x-axis.
+
+    Returns: Rotated component.
+    """
+    c = gf.Component()
+    component = gf.get_component(component)
+    ref = c.add_ref(component)
+    if x_mirror:
+        ref.mirror_x()
+    else:
+        ref.mirror_y()
+    c.add_ports(ref.ports)
+    c.copy_child_info(component)
+    return c
+
+
 if __name__ == "__main__":
-    c = gf.components.ring_single()
-    c = rotate(c, -90)
-    # c = gf.components.straight_pin(length=11, taper=None)
-    # c.trim(left=0, right=10, bottom=0, top=10)
-    # c = trim(c, domain=[[0, 0], [0, 10], [10, 10], [10, 0]])
+    c = gf.c.bend_circular()
+    c = mirror(c)
     c.show()
-
-    # c = gf.c.rectangle(size=(10, 10), centered=True)
-    # p = c.get_polygons(layers=("WG",), by="tuple")
-    # print(list(p.keys())[0])
-
-    # c = gf.Component()
-    # ref = c << gf.components.mzi_lattice()
-    # ref.dmovey(15)
-    # p = get_polygons_points(ref)
-    # p = get_point_inside(ref, layer=(1, 0))
-    # c.add_label(text="hello", position=p)
-    # c.show()
-    # c = move_port_to_zero(c, port_name="e4")
-    # c.show()
