@@ -1,4 +1,3 @@
-# type: ignore
 """Extract netlist from component port connectivity.
 
 Assumes two ports are connected when they have same width, x, y
@@ -20,54 +19,68 @@ Assumes two ports are connected when they have same width, x, y
 from __future__ import annotations
 
 from collections import defaultdict
-from collections.abc import Callable
+from collections.abc import Callable, Sequence
+from hashlib import md5
 from pprint import pprint
-from typing import Any
+from typing import Any, Protocol
 from warnings import warn
 
 import numpy as np
+from kfactory import DInstance, LayerEnum, ProtoTKCell, VInstance, VKCell
+from kfactory.kcell import AnyKCell, TKCell
 
-from gdsfactory import Port
-from gdsfactory.component import Component, ComponentReference
+from gdsfactory import Port, typings
+from gdsfactory.component import (
+    Component,
+    ComponentAllAngle,
+    ComponentReference,
+)
 from gdsfactory.name import clean_name
 from gdsfactory.serialization import clean_dict, clean_value_json
 from gdsfactory.typings import LayerSpec
 
 
-def _nets_to_connections(nets: list[dict], connections: dict) -> dict[str, str]:
+def nets_to_connections(
+    nets: list[dict[str, Any]], connections: dict[str, Any]
+) -> dict[str, str]:
+    # Use the given connections; create a shallow copy to avoid mutating the input.
     connections = dict(connections)
-    inverse_connections = {v: k for k, v in connections.items()}
 
-    def _is_connected(p: str) -> bool:
-        return (p in connections) or (p in inverse_connections)
-
-    def _add_connection(p: str, q: str) -> None:
-        connections[p] = q
-        inverse_connections[q] = p
-
-    def _get_connected_port(p: str) -> str:
-        return connections[p] if p in connections else inverse_connections[p]
+    # Flat set of all used ports for O(1) membership check.
+    used = set(connections.keys())
+    used.update(connections.values())
 
     for net in nets:
         p = net["p1"]
         q = net["p2"]
-        if _is_connected(p):
-            _q = _get_connected_port(p)
+        if p in used:
+            # Find the already connected q (if any)
+            _q = (
+                connections[p]
+                if p in connections
+                else next(k for k, v in connections.items() if v == p)
+            )
             raise ValueError(
                 "SAX currently does not support multiply connected ports. "
                 f"Got {p}<->{q} and {p}<->{_q}"
             )
-        if _is_connected(q):
-            _p = _get_connected_port(q)
+        if q in used:
+            _p = (
+                connections[q]
+                if q in connections
+                else next(k for k, v in connections.items() if v == q)
+            )
             raise ValueError(
                 "SAX currently does not support multiply connected ports. "
                 f"Got {p}<->{q} and {_p}<->{q}"
             )
-        _add_connection(p, q)
+        connections[p] = q
+        used.add(p)
+        used.add(q)
     return connections
 
 
-def get_default_connection_validators() -> dict[str, Callable]:
+def get_default_connection_validators() -> dict[str, Callable[..., None]]:
     return {"optical": validate_optical_connection, "electrical": _null_validator}
 
 
@@ -79,7 +92,9 @@ def get_instance_name_from_alias(reference: ComponentReference) -> str:
     Args:
         reference: reference that needs naming.
     """
-    return clean_name(reference.name)
+    name = reference.name or md5(str(reference).encode()).hexdigest()[:8]
+
+    return clean_name(name)
 
 
 def get_instance_name_from_label(
@@ -99,9 +114,10 @@ def get_instance_name_from_label(
     from gdsfactory.pdk import get_layer
 
     layer_label = get_layer(layer_label)
+    layer = layer_label[0] if isinstance(layer_label, LayerEnum) else layer_label
 
-    x = reference.dx
-    y = reference.dy
+    x = reference.x
+    y = reference.y
     labels = component.labels
 
     # default instance name follows component.aliases
@@ -111,27 +127,66 @@ def get_instance_name_from_label(
     for label in labels:
         xl = label.dposition[0]
         yl = label.dposition[1]
-        if x == xl and y == yl and label.layer == layer_label[0]:
+        if x == xl and y == yl and label.layer == layer:
             # print(label.text, xl, yl, x, y)
-            return label.text
+            return str(label.text)
 
     return text
 
 
-def _is_array_reference(ref: ComponentReference) -> bool:
-    return ref.na > 1 or ref.nb > 1
+def _is_array_reference(ref: DInstance | VInstance) -> bool:
+    # Direct attribute access is much faster than hasattr(),
+    # and in the common case (attributes present) the try branch is very fast.
+    try:
+        return ref.na > 1 or ref.nb > 1  # type: ignore[union-attr]
+    except AttributeError:
+        return False
+
+
+def _is_orthogonal_array_reference(ref: ComponentReference) -> bool:
+    # Store intermediate attributes to local variables for faster lookup
+    a, b = ref.a, ref.b
+    ay = a.y
+    if abs(ay) != 0:
+        return False
+    bx = b.x
+    return abs(bx) == 0
+
+
+def _has_ports_on_same_location(reference: VInstance | DInstance) -> bool:
+    """Check if a reference has any ports on the same location.
+
+    Args:
+        reference: ComponentReference to check.
+
+    Returns:
+        True if any ports are on the same location, False otherwise.
+    """
+    if _is_array_reference(reference):
+        # For array references, check each instance
+        for ia in range(reference.na):  # type: ignore[union-attr]
+            for ib in range(reference.nb):  # type: ignore[union-attr]
+                port_locations = set()
+                for port in reference.cell.ports:
+                    ref_port = reference.ports[port.name, ia, ib]  # type: ignore[index]
+                    port_loc = ref_port.to_itype().center
+                    if port_loc in port_locations:
+                        return True
+                    port_locations.add(port_loc)
+    else:
+        # For single references, check all ports
+        port_locations = set()
+        for port in reference.ports:
+            port_loc = port.to_itype().center
+            if port_loc in port_locations:
+                return True
+            port_locations.add(port_loc)
+    return False
 
 
 def get_netlist(
-    component: Component,
-    exclude_port_types: list[str] | tuple[str] | None = (
-        "placement",
-        "pad",
-        "bump",
-        "vertical_te",
-        "vertical_tm",
-        "edge_coupler",
-    ),
+    component: ProtoTKCell[Any] | VKCell | TKCell,
+    exclude_port_types: Sequence[str] | None = ("placement", "pad", "bump"),
     get_instance_name: Callable[..., str] = get_instance_name_from_alias,
     allow_multiple: bool = True,
     connection_error_types: dict[str, list[str]] | None = None,
@@ -164,31 +219,46 @@ def get_netlist(
         warnings: warning messages (disconnected pins).
 
     """
-    placements = {}
-    instances = {}
-    nets = []
-    top_ports = {}
+    if isinstance(component, VKCell):
+        component_: Component | ComponentAllAngle = ComponentAllAngle(
+            base=component.base
+        )
+    elif isinstance(component, ProtoTKCell):
+        component_ = Component(base=component.base)
+    else:
+        component_ = Component(base=component)
+
+    placements: dict[str, dict[str, Any]] = {}
+    instances: dict[str, dict[str, Any]] = {}
+    nets: list[dict[str, Any]] = []
+    top_ports: dict[str, str] = {}
 
     # store where ports are located
-    name2port = {}
+    name2port: dict[str, typings.Port] = {}
 
     # TOP level ports
-    ports = component.ports
-    ports_by_type = defaultdict(list)
-    top_ports_list = set()
+    ports = component_.ports
+    ports_by_type: defaultdict[str, list[str]] = defaultdict(list)
+    top_ports_list: set[str] = set()
 
-    references = _get_references_to_netlist(component)
+    if isinstance(component_, ProtoTKCell):
+        references: list[DInstance | VInstance] | list[VInstance] = (
+            _get_references_to_netlist(component_)
+        )
+    else:
+        references = _get_references_to_netlist_all_angle(component_)
 
     for reference in references:
+        # Skip references with ports on the same location
+        if _has_ports_on_same_location(reference):
+            continue
+
         c = reference.cell
-        origin = reference.dtrans.disp
+        origin = reference.dcplx_trans.disp
         x = origin.x
         y = origin.y
         reference_name = get_instance_name(reference)
-        is_array_ref = False
-        if _is_array_reference(reference):
-            is_array_ref = True
-        instance = {}
+        instance: dict[str, Any] = {}
 
         if c.info:
             instance.update(component=c.name, info=c.info.model_dump())
@@ -207,43 +277,53 @@ def get_netlist(
             )
 
         instances[reference_name] = instance
+
         placements[reference_name] = {
             "x": x,
             "y": y,
             "rotation": reference.dcplx_trans.angle,
-            "mirror": reference.dtrans.mirror,
+            "mirror": reference.dcplx_trans.mirror,
         }
 
-        if is_array_ref:
-            instances[reference_name].update(
-                columns=reference.na,
-                rows=reference.nb,
-                column_pitch=reference.da.x,
-                row_pitch=reference.db.y,
-            )
+        if _is_array_reference(reference):
+            if _is_orthogonal_array_reference(reference):  # type: ignore[arg-type]
+                instances[reference_name]["array"] = {
+                    "columns": reference.na,  # type: ignore[union-attr]
+                    "rows": reference.nb,  # type: ignore[union-attr]
+                    "column_pitch": reference.instance.da.x,  # type: ignore[union-attr]
+                    "row_pitch": reference.instance.db.y,  # type: ignore[union-attr]
+                }
+            else:
+                instances[reference_name]["array"] = {
+                    "num_a": reference.na,  # type: ignore[union-attr]
+                    "num_b": reference.nb,  # type: ignore[union-attr]
+                    "pitch_a": (reference.instance.da.x, reference.instance.da.y),  # type: ignore[union-attr]
+                    "pitch_b": (reference.instance.db.x, reference.instance.db.y),  # type: ignore[union-attr]
+                }
             reference_name = get_instance_name(reference)
-            for ia in range(reference.na):
-                for ib in range(reference.nb):
+            for ia in range(reference.na):  # type: ignore[union-attr]
+                for ib in range(reference.nb):  # type: ignore[union-attr]
                     for port in reference.cell.ports:
-                        ref_port = reference.ports[port.name, ia, ib]
+                        ref_port = reference.ports[port.name, ia, ib]  # type: ignore[index]
                         src = f"{reference_name}<{ia}.{ib}>,{port.name}"
                         name2port[src] = ref_port
                         ports_by_type[port.port_type].append(src)
         else:
             # lower level ports
-            for port in reference.ports:
+            for port_ in reference.ports:
                 reference_name = get_instance_name(reference)
-                src = f"{reference_name},{port.name}"
-                name2port[src] = port
-                ports_by_type[port.port_type].append(src)
+                src = f"{reference_name},{port_.name}"
+                name2port[src] = port_
+                ports_by_type[port_.port_type].append(src)
 
     for port in ports:
-        src = port.name
-        name2port[src] = port
-        top_ports_list.add(src)
-        ports_by_type[port.port_type].append(src)
+        port_name = port.name
+        if port_name is not None:
+            name2port[port_name] = port
+            top_ports_list.add(port_name)
+            ports_by_type[port.port_type].append(port_name)
 
-    warnings = {}
+    warnings: dict[str, Any] = {}
     for port_type, port_names in ports_by_type.items():
         if exclude_port_types and port_type in exclude_port_types:
             continue
@@ -256,7 +336,6 @@ def get_netlist(
         )
         if warnings_t:
             warnings[port_type] = warnings_t
-
         for connection in connections_t:
             if len(connection) == 2:
                 src, dst = connection
@@ -273,23 +352,23 @@ def get_netlist(
     nets_sorted = sorted(nets, key=lambda net: f"{net['p1']},{net['p2']}")
     placements_sorted = {k: placements[k] for k in sorted(placements.keys())}
     instances_sorted = {k: instances[k] for k in sorted(instances.keys())}
-    netlist = {
+    netlist: dict[str, Any] = {
         "nets": nets_sorted,
         "instances": instances_sorted,
         "placements": placements_sorted,
         "ports": top_ports,
-        "name": component.name,
+        "name": component_.name,
     }
     if warnings:
         netlist["warnings"] = warnings
-    return clean_value_json(netlist)
+    return clean_value_json(netlist)  # type: ignore[no-any-return]
 
 
 def extract_connections(
-    port_names: list[str],
-    ports: dict[str, Port],
+    port_names: Sequence[str],
+    ports: dict[str, typings.Port],
     port_type: str,
-    validators: dict[str, Callable] | None = None,
+    validators: dict[str, Callable[..., None]] | None = None,
     allow_multiple: bool = True,
     connection_error_types: dict[str, list[str]] | None = None,
 ) -> tuple[list[list[str]], dict[str, list[dict[str, Any]]]]:
@@ -308,14 +387,14 @@ def extract_connections(
 
 
 def _extract_connections(
-    port_names: list[str],
-    ports: dict[str, Port],
+    port_names: Sequence[str],
+    ports: dict[str, typings.Port],
     port_type: str,
-    connection_validator: Callable,
+    connection_validator: Callable[..., None],
     raise_error_for_warnings: list[str] | None = None,
     allow_multiple: bool = True,
     connection_error_types: dict[str, list[str]] | None = None,
-) -> tuple[list[list[str]], dict[str, list[dict[str, Any]]]]:
+) -> tuple[list[list[str]], dict[str, list[Any]]]:
     """Extracts connections between ports.
 
     Args:
@@ -331,7 +410,7 @@ def _extract_connections(
     if connection_error_types is None:
         connection_error_types = DEFAULT_CRITICAL_CONNECTION_ERROR_TYPES
 
-    warnings = defaultdict(list)
+    warnings: defaultdict[str, list[Any]] = defaultdict(list)
     if raise_error_for_warnings is None:
         raise_error_for_warnings = connection_error_types.get(port_type, [])
 
@@ -342,7 +421,7 @@ def _extract_connections(
 
     for port_name in unconnected_port_names:
         port = ports[port_name]
-        by_xy[port.center].append(port_name)
+        by_xy[port.to_itype().center].append(port_name)
 
     unconnected_port_names = []
 
@@ -358,7 +437,7 @@ def _extract_connections(
 
         elif not allow_multiple:
             warnings["multiple_connections"].append(ports_at_xy)
-            warn(f"Found multiple connections at {xy}:{ports_at_xy}")
+            warn(f"Found multiple connections at {xy}:{ports_at_xy}", stacklevel=3)
 
         else:
             # Iterates over the list of multiple ports to create related two-port connectivity
@@ -393,7 +472,7 @@ def _extract_connections(
 
     if critical_warnings and raise_error_for_warnings:
         pprint(critical_warnings)
-        warn("Found critical warnings while extracting netlist")
+        warn("Found critical warnings while extracting netlist", stacklevel=3)
     return connections, dict(warnings)
 
 
@@ -427,7 +506,9 @@ def validate_optical_connection(
     is_top_level = [("," not in pname) for pname in port_names]
 
     if len(port_names) != 2:
-        raise ValueError(f"More than two connected optical ports: {port_names}")
+        raise ValueError(
+            f"More than two connected optical ports: {port_names} at {port1.center}"
+        )
 
     if all(is_top_level):
         raise ValueError(f"Two top-level ports appear to be connected: {port_names}")
@@ -489,14 +570,27 @@ def difference_between_angles(angle2: float, angle1: float) -> float:
     return diff
 
 
-def _get_references_to_netlist(component: Component) -> list[ComponentReference]:
-    return component.insts
+def _get_references_to_netlist(component: Component) -> list[DInstance | VInstance]:
+    insts = component.insts
+    vinsts = component.vinsts
+    return list(insts) + list(vinsts)
+
+
+def _get_references_to_netlist_all_angle(
+    component: ComponentAllAngle,
+) -> list[VInstance]:
+    insts = list(component.insts)
+    return insts
+
+
+class GetNetlistFunc(Protocol):
+    def __call__(self, component: AnyKCell, **kwargs: Any) -> dict[str, Any]: ...
 
 
 def get_netlist_recursive(
-    component: Component,
+    component: AnyKCell,
     component_suffix: str = "",
-    get_netlist_func: Callable = get_netlist,
+    get_netlist_func: GetNetlistFunc = get_netlist,  # type: ignore[assignment]
     get_instance_name: Callable[..., str] = get_instance_name_from_alias,
     **kwargs: Any,
 ) -> dict[str, Any]:
@@ -519,10 +613,17 @@ def get_netlist_recursive(
         Dictionary of netlists, keyed by the name of each component.
 
     """
-    all_netlists = {}
+    all_netlists: dict[str, Any] = {}
 
     # only components with references (subcomponents) warrant a netlist
-    references = _get_references_to_netlist(component)
+    if isinstance(component, ProtoTKCell):
+        component = Component(base=component.base)
+        references: list[DInstance | VInstance] | list[VInstance] = (
+            _get_references_to_netlist(component)
+        )
+    else:
+        component = ComponentAllAngle(base=component.base)
+        references = _get_references_to_netlist_all_angle(component)
 
     if references:
         netlist = get_netlist_func(component, **kwargs)
@@ -539,11 +640,20 @@ def get_netlist_recursive(
             )
             all_netlists |= grandchildren
 
-            child_references = _get_references_to_netlist(ref.cell)
+            if isinstance(ref.cell, ProtoTKCell):
+                child_references: list[VInstance | DInstance] | list[VInstance] = (
+                    _get_references_to_netlist(Component(base=ref.cell.base))
+                )
+            else:
+                child_references = _get_references_to_netlist_all_angle(
+                    ComponentAllAngle(base=ref.cell)  # type: ignore[call-overload]
+                )
 
             if child_references:
                 inst_name = get_instance_name(ref)
-                netlist_dict = {"component": f"{rcell.name}{component_suffix}"}
+                netlist_dict: dict[str, Any] = {
+                    "component": f"{rcell.name}{component_suffix}"
+                }
                 if hasattr(rcell, "settings"):
                     netlist_dict.update(settings=rcell.settings.model_dump())
                 if hasattr(rcell, "info"):
@@ -554,9 +664,9 @@ def get_netlist_recursive(
 
 
 def _demo_ring_single_array() -> None:
-    import gdsfactory as gf
+    from gdsfactory.components.rings.ring_single_array import ring_single_array
 
-    c = gf.components.ring_single_array()
+    c = ring_single_array()
     c.get_netlist()
 
 
@@ -589,7 +699,7 @@ if __name__ == "__main__":
 
     # c = gf.Component()
     # mzi = c << gf.c.mzi()
-    # mzi.dxmin = 10
+    # mzi.xmin = 10
     # mzi.name = "mzi"
     # bend = c << gf.c.bend_euler()
     # bend.connect("o1", mzi.ports["o2"])
