@@ -14,20 +14,11 @@ import numpy as np
 
 import gdsfactory as gf
 from gdsfactory.component import Component
-from gdsfactory.components.rectangle import rectangle
-from gdsfactory.components.text_rectangular import text_rectangular
-from gdsfactory.components.triangles import triangle
-from gdsfactory.typings import (
-    Anchor,
-    ComponentSpec,
-    ComponentSpecsOrComponents,
-    Float2,
-    Spacing,
-)
+from gdsfactory.typings import Anchor, ComponentSpec, ComponentSpecs, Float2, Spacing
 
 
 def grid(
-    components: ComponentSpecsOrComponents = (rectangle, triangle),
+    components: ComponentSpecs = ("rectangle", "triangle"),
     spacing: Spacing | float = (5.0, 5.0),
     shape: tuple[int, int] | None = None,
     align_x: Literal["origin", "xmin", "xmax", "center"] = "center",
@@ -61,8 +52,7 @@ def grid(
             components,
             shape=(1, len(components)),
             rotation=0,
-            h_mirror=False,
-            v_mirror=True,
+            mirror=False,
             spacing=(100, 100),
         )
         c.plot()
@@ -80,31 +70,29 @@ def grid(
         ),
         align_x=align_x,
         align_y=align_y,
-        rotation=round(rotation // 90),  # type: ignore
+        rotation=rotation,
         mirror=mirror,
     )
-    for i, instances_list in enumerate(instances):
-        for j, instance in enumerate(instances_list):
-            if instance is not None:
-                c.add_ports(instance.ports, prefix=f"{j}_{i}_")
+    for i, instance in enumerate(instances):
+        c.add_ports(instance.ports, prefix=f"{i}_")
     return c
 
 
 def grid_with_text(
-    components: Sequence[ComponentSpec] = (rectangle, triangle),
+    components: Sequence[ComponentSpec] = ("rectangle", "triangle"),
     text_prefix: str = "",
-    text_offsets: tuple[Float2, ...] | None = None,
-    text_anchors: tuple[Anchor, ...] | None = None,
+    text_offsets: Sequence[Float2] | None = None,
+    text_anchors: Sequence[Anchor] | None = None,
     text_mirror: bool = False,
     text_rotation: int = 0,
-    text: ComponentSpec | None = text_rectangular,
+    text: ComponentSpec | None = "text_rectangular",
     spacing: Spacing | float = (5.0, 5.0),
     shape: tuple[int, int] | None = None,
     align_x: Literal["origin", "xmin", "xmax", "center"] = "center",
     align_y: Literal["origin", "ymin", "ymax", "center"] = "center",
     rotation: int = 0,
     mirror: bool = False,
-    labels: tuple[str, ...] | None = None,
+    labels: Sequence[str] | None = None,
 ) -> Component:
     """Returns Component with 1D or 2D grid of components with text labels.
 
@@ -136,8 +124,7 @@ def grid_with_text(
             components,
             shape=(1, len(components)),
             rotation=0,
-            h_mirror=False,
-            v_mirror=True,
+            mirror=False,
             spacing=(100, 100),
             text_offsets=((0, 100), (0, -100)),
             text_anchors=("nc", "sc"),
@@ -145,15 +132,22 @@ def grid_with_text(
         c.plot()
 
     """
-    components = [gf.get_component(component) for component in components]
-    text_offsets = text_offsets or [(0, 0)]
-    text_anchors = text_anchors or ["center"]
-    labels = labels or [None] * len(components)
+    component_list = [gf.get_component(component) for component in components]
+    text_offsets = text_offsets or ((0, 0),)
+    text_anchors = text_anchors or ("center",)
+    labels_not_none: list[str | None] = (
+        list(labels) if labels else [None] * len(component_list)
+    )
+
+    if len(labels_not_none) != len(component_list):
+        raise ValueError(
+            f"Number of labels {len(labels_not_none)} must match number of components {len(component_list)}"
+        )
 
     c = gf.Component()
     instances = kf.grid(
         c,
-        kcells=components,
+        kcells=component_list,
         shape=shape,
         spacing=(
             (float(spacing[0]), float(spacing[1]))
@@ -162,29 +156,26 @@ def grid_with_text(
         ),
         align_x=align_x,
         align_y=align_y,
-        rotation=round(rotation // 90),
+        rotation=rotation,
         mirror=mirror,
     )
-    for i, instances_list in enumerate(instances):
-        for j, instance in enumerate(instances_list):
-            if instance is None:
-                continue
-            c.add_ports(instance.ports, prefix=f"{j}_{i}_")
-            text_string = labels[i] or f"{text_prefix}{j}_{i}"
+    for i, instance in enumerate(instances):
+        c.add_ports(instance.ports, prefix=f"{i}_")
+        text_string = labels_not_none[i] or f"{text_prefix}_{i}"
 
-            if text:
-                for text_offset, text_anchor in zip_longest(text_offsets, text_anchors):
-                    t = c << text(text_string)
-                    size_info = instance.dsize_info
-                    text_offset = text_offset or (0, 0)
-                    text_anchor = text_anchor or "center"
-                    o = np.array(text_offset)
-                    d = np.array(getattr(size_info, text_anchor))
-                    t.dmove(o + d)
-                    if text_mirror:
-                        t.dmirror()
-                    if text_rotation:
-                        t.drotate(text_rotation)
+        if text:
+            for text_offset, text_anchor in zip_longest(text_offsets, text_anchors):
+                t = c << gf.get_component(text, text=text_string)
+                size_info = instance.dsize_info
+                text_offset = text_offset or (0, 0)
+                text_anchor = text_anchor or "center"
+                o = np.array(text_offset)
+                d = np.array(getattr(size_info, text_anchor))
+                t.move(tuple(o + d))
+                if text_mirror:
+                    t.dmirror()
+                if text_rotation:
+                    t.rotate(text_rotation)
     return c
 
 
@@ -194,18 +185,18 @@ if __name__ == "__main__":
     # test_grid()
     # components = [gf.components.rectangle(size=(i, i)) for i in range(40, 66, 5)]
     # c = tuple(gf.components.rectangle(size=(i, i)) for i in range(40, 66, 10))
-    # c = tuple([gf.components.triangle(x=i) for i in range(1, 10)])
-    c = tuple(gf.components.rectangle(size=(i, i)) for i in range(1, 3))
+    components = tuple([gf.components.triangle(x=i) for i in range(1, 10)])
+    # components = tuple(gf.components.rectangle(size=(i, i)) for i in range(1, 3))
     # print(len(c))
 
-    c = grid_with_text(
-        c,
+    c = grid(
+        components,
         shape=(3, 3),
-        # rotation=90,
+        rotation=90,
         mirror=False,
         spacing=(200.0, 200.0),
         # spacing=1,
-        text_offsets=((0, 100), (0, -100)),
-        labels=["r1", "r2"],
+        # text_offsets=((0, 100), (0, -100)),
+        # labels=["r1", "r2"],
     )
     c.show()
