@@ -2,7 +2,7 @@
 
 from __future__ import annotations
 
-from typing import Any
+from typing import Any, cast
 
 import numpy as np
 import numpy.typing as npt
@@ -10,6 +10,7 @@ import numpy.typing as npt
 import gdsfactory as gf
 from gdsfactory.boolean import boolean
 from gdsfactory.component import Component
+from gdsfactory.typings import PathType
 
 
 def compute_area_signed(pr: npt.NDArray[np.floating[Any]]) -> float:
@@ -22,7 +23,9 @@ def compute_area_signed(pr: npt.NDArray[np.floating[Any]]) -> float:
     xs, ys = map(list, zip(*pr))
     xs.append(xs[1])
     ys.append(ys[1])
-    return sum(xs[i] * (ys[i + 1] - ys[i - 1]) for i in range(1, len(pr))) / 2.0
+    xs_ = cast(list[float], xs)
+    ys_ = cast(list[float], ys)
+    return sum(xs_[i] * (ys_[i + 1] - ys_[i - 1]) for i in range(1, len(pr))) / 2.0
 
 
 def from_np(
@@ -66,7 +69,7 @@ def from_np(
 
 
 @gf.cell
-def from_image(image_path: str, **kwargs: Any) -> Component:
+def from_image(image_path: PathType, **kwargs: Any) -> Component:
     """Returns Component from a png image.
 
     Args:
