@@ -1,17 +1,13 @@
 from __future__ import annotations
 
-import warnings
 from collections.abc import Mapping, Sequence
 from typing import Any
 
 import kfactory as kf
+import klayout.db as kdb
 
 import gdsfactory as gf
 from gdsfactory.component import Component, ComponentReference
-from gdsfactory.components.bend_euler import bend_euler
-from gdsfactory.components.grating_coupler_elliptical_trenches import grating_coupler_te
-from gdsfactory.components.straight import straight as straight_function
-from gdsfactory.cross_section import strip
 from gdsfactory.port import select_ports_optical
 from gdsfactory.routing.route_bundle import get_min_spacing, route_bundle
 from gdsfactory.routing.route_single import route_single
@@ -31,9 +27,9 @@ def route_fiber_array(
     component: Component,
     component_to_route: Component | ComponentReference,
     pitch: float = 127.0,
-    grating_coupler: ComponentSpecOrList = grating_coupler_te,
-    bend: ComponentSpec = bend_euler,
-    straight: ComponentSpec = straight_function,
+    grating_coupler: ComponentSpecOrList = "grating_coupler_te",
+    bend: ComponentSpec = "bend_euler",
+    straight: ComponentSpec = "straight",
     fanout_length: float | None = None,
     max_y0_optical: None = None,
     with_loopback: bool = True,
@@ -53,10 +49,10 @@ def route_fiber_array(
     select_ports: PortsFactory = select_ports_optical,
     radius: float | None = None,
     radius_loopback: float | None = None,
-    cross_section: CrossSectionSpec = strip,
+    cross_section: CrossSectionSpec = "strip",
     allow_width_mismatch: bool = False,
     port_type: str = "optical",
-    route_width: float | list[float] | None = 0,
+    route_width: float | None = None,
     start_straight_length: float = 0,
     end_straight_length: float = 0,
     auto_taper: bool = True,
@@ -64,7 +60,6 @@ def route_fiber_array(
     steps: Sequence[Mapping[str, int | float]] | None = None,
     bboxes: BoundingBoxes | None = None,
     avoid_component_bbox: bool = True,
-    fiber_spacing: None | float = None,
     **kwargs: Any,
 ) -> Component:
     """Returns new component with fiber array.
@@ -116,26 +111,17 @@ def route_fiber_array(
         steps: steps for the route.
         bboxes: list bounding boxes to avoid for routing.
         avoid_component_bbox: avoid component bbox for routing.
-        fiber_spacing: Deprecated. Use pitch instead.
         kwargs: route_bundle settings.
     """
-    c = component
-    component = component_to_route
     x = gf.get_cross_section(cross_section)
 
-    if fiber_spacing is not None:
-        warnings.warn(
-            "fiber_spacing is deprecated. Use pitch instead", DeprecationWarning
-        )
-        pitch = fiber_spacing
-
-    component_name = component_name or component.name
+    component_name = component_name or component_to_route.name
     excluded_ports = excluded_ports or []
     if port_names is None:
-        to_route = list(select_ports(component.ports))
-        port_names = [p.name for p in to_route]
+        to_route = list(select_ports(component_to_route.ports))
+        port_names = [p.name for p in to_route if p.name is not None]
     else:
-        to_route = [component.ports[lbl] for lbl in port_names]
+        to_route = [component_to_route.ports[lbl] for lbl in port_names]
 
     to_route = [p for p in to_route if p.name not in excluded_ports]
 
@@ -148,7 +134,7 @@ def route_fiber_array(
     # optical_ports_labels = [p.name for p in ports]
     # print(optical_ports_labels)
     if n == 0:
-        return [], [], 0
+        return component
 
     # grating_coupler can either be a component/function or a list of components/functions
     if isinstance(grating_coupler, list):
@@ -174,11 +160,11 @@ def route_fiber_array(
 
     # `delta_gr_min` Used to avoid crossing between straights in special cases
     # This could happen when abs(x_port - x_grating) <= 2 * radius
-    dy = bend90.dysize
+    dy = bend90.ysize
     delta_gr_min = 2 * dy + 1
 
     # Get the center along x axis
-    x_c = round(sum(p.dx for p in to_route) / n, 1)
+    x_c = round(sum(p.x for p in to_route) / n, 1)
 
     # Sort the list of optical ports:
     direction_ports = direction_ports_from_list_ports(to_route)
@@ -188,11 +174,11 @@ def route_fiber_array(
     k = k + 1 if k % 2 else k
 
     # Set routing type if not specified
-    pxs = [p.dx for p in to_route]
+    pxs = [p.x for p in to_route]
     is_big_component = (
         (k > 2)
         or (max(pxs) - min(pxs) > pitch - delta_gr_min)
-        or (component.dxsize > pitch)
+        or (component_to_route.xsize > pitch)
     )
 
     def has_p(side: str) -> bool:
@@ -231,7 +217,10 @@ def route_fiber_array(
 
     # use x for grating coupler since we rotate it
     y0_optical = (
-        component.dymin - fanout_length - grating_coupler.ports[gc_port_name].dx - dy
+        component_to_route.ymin
+        - fanout_length
+        - grating_coupler.ports[gc_port_name].x
+        - dy
     )
     y0_optical += -k / 2 * separation
 
@@ -244,7 +233,7 @@ def route_fiber_array(
     # - then connect south ports (left to right)
     # - then east ports (bottom to top)
     # - then second half of the north ports (right to left)
-    ports = []
+    ports: list[gf.Port] = []
     north_ports = direction_ports["N"]
     north_start = north_ports[: len(north_ports) // 2]
     north_finish = north_ports[len(north_ports) // 2 :]
@@ -259,9 +248,11 @@ def route_fiber_array(
 
     nb_ports_per_line = n // nb_optical_ports_lines
     y_gr_gap = (k / nb_optical_ports_lines + 1) * separation
-    gr_coupler_y_sep = grating_coupler.dysize + y_gr_gap + dy
+    gr_coupler_y_sep = grating_coupler.ysize + y_gr_gap + dy
     offset = (nb_ports_per_line - 1) * pitch / 2 - x_grating_offset
-    io_gratings_lines = []  # [[gr11, gr12, gr13...], [gr21, gr22, gr23...] ...]
+    io_gratings_lines: list[
+        list[Component | kf.DInstance]
+    ] = []  # [[gr11, gr12, gr13...], [gr21, gr22, gr23...] ...]
 
     grating_coupler_port_names = [p.name for p in grating_coupler.ports]
     with_fiber_port = gc_port_name_fiber in grating_coupler_port_names
@@ -272,14 +263,14 @@ def route_fiber_array(
         assert len(grating_indices) == nb_ports_per_line
 
     # add grating couplers
-    io_gratings = []
-    gc_ports = []
+    io_gratings: list[Component | kf.DInstance] = []
+    gc_ports: list[gf.Port] = []
     for j in range(nb_optical_ports_lines):
         for i, gc in zip(grating_indices, grating_couplers):
-            gc_ref = c << gc
-            gc_ref.drotate(gc_rotation)
-            gc_ref.dx = x_c - offset + i * pitch
-            gc_ref.dymax = y0_optical - j * gr_coupler_y_sep
+            gc_ref = component << gc
+            gc_ref.rotate(gc_rotation)
+            gc_ref.x = x_c - offset + i * pitch
+            gc_ref.ymax = y0_optical - j * gr_coupler_y_sep
             io_gratings += [gc_ref]
 
         io_gratings_lines += [io_gratings[:]]
@@ -294,37 +285,38 @@ def route_fiber_array(
         for io_gratings in io_gratings_lines:
             for gr in io_gratings:
                 for p in to_route:
-                    dist = gr.dx - p.dx
+                    dist = gr.x - p.x
                     if abs(dist) < abs(min_dist):
                         min_dist = dist
             if abs(min_dist) < min_dist_threshold:
                 for gr in io_gratings:
-                    gr.dmovex(-min_dist)
+                    gr.movex(-min_dist)
 
     # If the array of gratings is too close, adjust its location
-    gc_ports_tmp = []
+    gc_ports_tmp: list[gf.Port] = []
     for io_gratings in io_gratings_lines:
         gc_ports_tmp += [gc.ports[gc_port_name] for gc in io_gratings]
     min_y = get_min_spacing(to_route, gc_ports_tmp, separation=separation, radius=dy)
-    delta_y = abs(to_route[0].dy - gc_ports_tmp[0].dy)
+    delta_y = abs(to_route[0].y - gc_ports_tmp[0].y)
 
     if min_y > delta_y:
         for io_gratings in io_gratings_lines:
             for gr in io_gratings:
-                gr.dy += delta_y - min_y
+                gr.y += delta_y - min_y
 
     # If we add align ports, we need enough space for the bends
     io_gratings = io_gratings_lines[0]
     gc_ports = [gc.ports[gc_port_name] for gc in io_gratings]
     # c.shapes(c.kcl.layer(1, 10)).insert(component_to_route.bbox())
 
-    bboxes = bboxes or []
+    _bboxes: list[kdb.DBox] = [kdb.DBox(*bbox) for bbox in bboxes or []]
 
     if avoid_component_bbox:
-        bboxes += [component_to_route.bbox()]
+        bbox = component_to_route.bbox()
+        _bboxes.append(kdb.DBox(bbox.left, bbox.bottom, bbox.right, bbox.top))
 
     route_bundle(
-        c,
+        component,
         ports1=list(to_route),
         ports2=list(gc_ports),
         separation=separation,
@@ -335,7 +327,7 @@ def route_fiber_array(
         sort_ports=True,
         allow_width_mismatch=allow_width_mismatch,
         route_width=route_width,
-        bboxes=bboxes,
+        bboxes=_bboxes,
         start_straight_length=start_straight_length,
         end_straight_length=end_straight_length,
         auto_taper=auto_taper,
@@ -344,80 +336,72 @@ def route_fiber_array(
         **kwargs,
     )
     if gc_port_name_fiber not in grating_coupler_port_names:
-        gc_port_name_fiber = gc_port_names[0]
+        _gc_port_name_fiber = gc_port_names[0]
+        if _gc_port_name_fiber is not None:
+            gc_port_name_fiber = _gc_port_name_fiber
+        else:
+            raise ValueError(
+                f"{gc_port_name_fiber!r} not in {grating_coupler_port_names}"
+            )
+
     fiber_ports = [gc.ports[gc_port_name_fiber] for gc in io_gratings]
 
-    c.ports = kf.Ports(kcl=c.kcl)
+    component.ports = kf.DPorts(kcl=component.kcl)
 
     for component_port, port in zip(port_names, fiber_ports):
-        c.add_port(name=component_port, port=port)
+        component.add_port(name=component_port, port=port)
 
-    c.add_ports(ports_not_terminated)
+    component.add_ports(ports_not_terminated)
     if with_loopback:
         ii = [grating_indices[0] - 1, grating_indices[-1] + 1]
-        gca1 = c << grating_coupler
-        gca2 = c << grating_coupler
-        gca1.drotate(gc_rotation)
-        gca2.drotate(gc_rotation)
+        gca1 = component << grating_coupler
+        gca2 = component << grating_coupler
+        gca1.rotate(gc_rotation)
+        gca2.rotate(gc_rotation)
 
-        gca1.dx = x_c - offset + ii[0] * pitch
-        gca2.dx = x_c - offset + ii[1] * pitch
+        gca1.x = x_c - offset + ii[0] * pitch
+        gca2.x = x_c - offset + ii[1] * pitch
 
-        gc_loopback_dymin = io_gratings_lines[0][0].dymin
-        gca1.dymin = gc_loopback_dymin
-        gca2.dymin = gc_loopback_dymin
+        gc_loopback_dymin = io_gratings_lines[0][0].ymin
+        gca1.ymin = gc_loopback_dymin
+        gca2.ymin = gc_loopback_dymin
 
         port0 = gca1[gc_port_name]
         port1 = gca2[gc_port_name]
         radius = radius_loopback or radius or x.radius
-        radius_dbu = c.kcl.to_dbu(radius)
-        d_loop = straight_to_grating_spacing + radius + gca1.dysize
-        d_loop_dbu = c.kcl.to_dbu(d_loop)
+        assert radius is not None
+        radius_dbu = component.kcl.to_dbu(radius)
+        d_loop = straight_to_grating_spacing + radius + gca1.ysize
+        d_loop_dbu = component.kcl.to_dbu(d_loop)
 
-        waypoints = kf.routing.optical.route_loopback(
-            port0,
-            port1,
+        waypoints_loopback = kf.routing.optical.route_loopback(
+            port0.to_itype(),
+            port1.to_itype(),
             bend90_radius=radius_dbu,
             inside=with_loopback_inside,
             d_loop=d_loop_dbu,
         )
+        waypoints_loopback_ = [
+            p.to_dtype(component.kcl.dbu) for p in waypoints_loopback
+        ]
         bend90 = gf.get_component(
             bend, cross_section=cross_section, radius=radius_loopback
         )
 
         route_single(
-            c,
+            component,
             port1=port0,
             port2=port1,
-            waypoints=waypoints,
+            waypoints=waypoints_loopback_,
             straight=straight,
             bend=bend90,
             cross_section=cross_section,
         )
         port0 = gca1[gc_port_name_fiber]
         port1 = gca2[gc_port_name_fiber]
-        c.add_port(name="loopback1", port=port0)
-        c.add_port(name="loopback2", port=port1)
-    return c
-
-
-def demo() -> None:
-    gcte = gf.components.grating_coupler_te
-    gctm = gf.components.grating_coupler_tm
-
-    c = gf.components.straight(length=500)
-    c = gf.components.mmi2x2()
-
-    route_fiber_array(
-        component=c,
-        grating_coupler=[gcte, gctm, gcte, gctm],
-        with_loopback=True,
-        # bend=gf.components.bend_euler,
-        bend=gf.components.bend_circular,
-        radius=20,
-        # force_manhattan=True
-    )
-    c.show()
+        component.add_port(name="loopback1", port=port0)
+        component.add_port(name="loopback2", port=port1)
+    return component
 
 
 if __name__ == "__main__":
