@@ -5,7 +5,7 @@ from gdsfactory.component import Component
 
 
 @gf.cell
-def test_netlist_yaml() -> Component:
+def netlist_yaml() -> Component:
     """Test netlist yaml.
 
     .. code::
@@ -40,6 +40,8 @@ def test_netlist_yaml() -> Component:
 
     routes:
         optical:
+            settings:
+                cross_section: strip
             links:
                 mmi_short,o2: mmi_long,o3
 
@@ -51,6 +53,10 @@ def test_netlist_yaml() -> Component:
     return gf.read.from_yaml(yaml)
 
 
+def test_netlist_yaml_sample() -> None:
+    assert netlist_yaml()
+
+
 if __name__ == "__main__":
-    c = test_netlist_yaml()
-    c.show(show_ports=True)
+    c = netlist_yaml()
+    c.show()
