@@ -36,7 +36,7 @@ def symbol_from_cell(func: _F, to_symbol: ToSymbol) -> _F:
         c_symbol = to_symbol(component, prefix=f"SYMBOL_{func.__name__}")
         return c_symbol
 
-    _symbol._symbol = True  # type: ignore
+    _symbol._symbol = True  # type: ignore[attr-defined]
     return _symbol
 
 
@@ -71,8 +71,8 @@ def floorplan_with_block_letters(
 
     # add floorplan box
     bbox = sym << rectangle(size=(w, h), layer=bbox_layer)
-    bbox.dx = component.dx
-    bbox.dy = component.dy
+    bbox.x = component.x
+    bbox.y = component.y
 
     # add text, fit to box with specified margin
     margin = 0.2
@@ -99,8 +99,8 @@ def floorplan_with_block_letters(
     )
 
     text = sym << text_component
-    text.dx = component.dx
-    text.dy = component.dy
+    text.x = component.x
+    text.y = component.y
 
     sym.add_ports(component.ports)
 
@@ -118,6 +118,6 @@ def floorplan_with_block_letters(
 if __name__ == "__main__":
     import gdsfactory as gf
 
-    c = gf.c.mmi1x2()  # type: ignore
+    c = gf.c.mmi1x2()
     s = floorplan_with_block_letters(c)
     s.show()
